import json
import os
import pprint
import pandas as pd
from fNIRS_BIDS import fNIRS_BIDS

fNIRS_BIdS_obj = fNIRS_BIDS()
dataset_path = 'BIDS-NIRS-Tapping'
metadata_validation_info = fNIRS_BIdS_obj.validate_fNIRS_BIDS_dataset(dataset_path)
sys.stdout.write(json.dumps(metadata_validation_info))
# pp = pprint.PrettyPrinter(indent=4)
# pp.pprint(metadata_validation_info)


import json
import os
import pprint
import pandas as pd

class fNIRS_BIDS():
    
    def __init__(self):
        f = open('BIDS_raw_folder.json')
        self.raw_folder_struture = json.load(f)
        
        f = open('BIDS_fNIRS_subject_folder.json')
        self.sub_folder_struture = json.load(f)
        
    def get_key_pos(self,key,file_folder_list):
            pos = -1
            for index, file_or_folder in enumerate(file_folder_list):
                if file_or_folder.endswith(key):
                    pos = index
                    break
                    
            return pos
        
    def get_key_path(self,key,file_folder_list):
        path = ''
        for index, file_or_folder in enumerate(file_folder_list):
            if file_or_folder.endswith(key):
                path = file_folder_list[index]
                break

        return path
        
    def get_key_paths(self, key, datset_path,sub_or_ses_folder):
        sub_ses_split = os.path.split(sub_or_ses_folder)
        paths = []
        if not sub_ses_split[0] == '':
            file_folder_list = os.listdir(os.path.join(datset_path,sub_ses_split[0]))
            path = self.get_key_path(key,file_folder_list)
            if path:
                paths.append(os.path.join(datset_path,sub_ses_split[0],path))
                        
        file_folder_list = os.listdir(os.path.join(datset_path,sub_or_ses_folder))
        path = self.get_key_path(key,file_folder_list)
        if path:
            paths.append(os.path.join(datset_path,sub_or_ses_folder,path))
                        
        file_folder_list = os.listdir(os.path.join(datset_path,sub_or_ses_folder,'nirs'))
        path = self.get_key_path(key,file_folder_list)
        if path:
            paths.append(os.path.join(datset_path,sub_or_ses_folder,'nirs',path))
                        
        return paths
        
        
    def validate_fNIRS_BIDS_dataset(self, datset_path):
        
        files_and_folders = os.listdir(datset_path)
        sub_folders = []
        
        # removes extensions for filenames for files othan than .json or .tsv files
        for index, file_or_folder in enumerate(files_and_folders):
            
            if os.path.isdir(os.path.join(datset_path, file_or_folder)):
                if file_or_folder.startswith('sub-'):
                    sub_folders.append(file_or_folder)
            else:
                ext = os.path.splitext(file_or_folder)[1]
                if not (ext == '.json' or ext == '.tsv'):
                    files_and_folders[index] = os.path.splitext(file_or_folder)[0]
                            
        
        raw_folder_keys = list(self.raw_folder_struture.keys())
        
        metadata_validation_info = {}
        for key in raw_folder_keys:
            key_ext = os.path.splitext(key)[1]
            if key in files_and_folders:
                ext = os.path.splitext(key)[1]
                temp_dict = {}
                __RequirementLevel__ = self.raw_folder_struture[key]['__RequirementLevel__']
                
                
                if ext == '.json':
                    f = open(os.path.join(datset_path, key))
                    json_data = json.load(f)
                    json_data_keys = list(json_data.keys())
                    sub_keys = list(self.raw_folder_struture[key].keys())
                    for sub_key in sub_keys:
                        if not sub_key == '__RequirementLevel__':
                            if not sub_key in json_data_keys:
                                temp_dict[sub_key] = {'__RequirementLevel__':self.raw_folder_struture[key][sub_key],
                                                      '__Value__':'MISSING'}
                                
                elif ext == '.tsv':
                    df = pd.read_csv(os.path.join(datset_path, key), sep='\t', keep_default_na=False, na_values=['',None])
                    sub_keys = list(self.raw_folder_struture[key].keys())
                    df_keys = list(df.keys())
                    for sub_key in sub_keys:
                        if not sub_key == '__RequirementLevel__':
                            if not sub_key in df_keys:
                                temp_dict[sub_key] = {'__RequirementLevel__':self.raw_folder_struture[key][sub_key],
                                                      '__Value__':'MISSING'}
                if temp_dict:          
                    temp_dict['__RequirementLevel__'] = __RequirementLevel__
                    temp_dict['__Value__'] = 'INVALID'
                    metadata_validation_info[key] = temp_dict
                
            else:
                __RequirementLevel__ = self.raw_folder_struture[key]['__RequirementLevel__']
                metadata_validation_info[key] = {'__RequirementLevel__' : __RequirementLevel__,
                                                       '__Value__' : 'MISSING'
                                                      }
        
        
        for sub_folder in sub_folders:
            BIDS_fNIRS_ses_folder_missing_metadata_list = []
            sub_folder_files_and_folders = os.listdir(os.path.join(datset_path,sub_folder))
            sub_or_ses_folders = []
            for sub_folder_file_and_folder in sub_folder_files_and_folders:
                if os.path.isdir(os.path.join(datset_path, sub_folder, sub_folder_file_and_folder)):
                    if sub_folder_file_and_folder.startswith('ses-'):
                        sub_or_ses_folders.append(sub_folder_file_and_folder)
                        
            if not sub_or_ses_folders:
                sub_or_ses_folders = [sub_folder]
            else:
                for index, value in enumerate(sub_or_ses_folders):
                    sub_or_ses_folders[index] = os.path.join(sub_folder, value)
                                
            for sub_or_ses_folder in sub_or_ses_folders:
                BIDS_sub_folder_missing_metadata = {}
                sub_or_ses_folder_files_and_folders = os.listdir(os.path.join(datset_path,sub_or_ses_folder,'nirs'))
                sub_folder_keys = list(self.sub_folder_struture.keys())
                
                for key in sub_folder_keys:
                    pos = self.get_key_pos(key,sub_or_ses_folder_files_and_folders)
                    paths = self.get_key_paths(key, datset_path, sub_or_ses_folder)

                    ###### This needs to be updated to get exact file location #####################
                    key_name = os.path.join(sub_or_ses_folder, os.path.basename(paths[0]))
                    if  paths: 
                        ext = os.path.splitext(key)[1]
                        temp_dict = {}
                        __RequirementLevel__ = self.sub_folder_struture[key]['__RequirementLevel__']
                        
                        if ext == '.json':
                            sub_keys = list(self.sub_folder_struture[key].keys())
                            json_data_keys = []
                            for u in range(len(paths)):
                                f = open(paths[u])
                                json_data = json.load(f)
                                json_data_keys.append(list(json_data.keys()))
    
                            json_data_keys = [x for sublist in json_data_keys for x in sublist]
                                                        
                            for sub_key in sub_keys:
                                if not sub_key == '__RequirementLevel__':
                                    if not sub_key in json_data_keys:
                                        temp_dict[sub_key] = {'__RequirementLevel__':self.sub_folder_struture[key][sub_key],
                                                             '__Value__': 'MISSING'}

                        elif ext == '.tsv':
                            df_keys = []
                            for u in range(len(paths)):
                                df = pd.read_csv(paths[u], sep='\t', keep_default_na=False, na_values=['',None])
                                df_keys.append(list(df.keys()))
    
                            df_keys = [x for sublist in df_keys for x in sublist]
                            
                            sub_keys = list(self.sub_folder_struture[key].keys())
                            for sub_key in sub_keys:
                                if not sub_key == '__RequirementLevel__':
                                    if not sub_key in df_keys:
                                        temp_dict[sub_key] = {'__RequirementLevel__':self.sub_folder_struture[key][sub_key],
                                                             '__Value__': 'MISSING'}
                                    else:
                                        row_numbers = df[df[sub_key].isna()].index.format()
                                        temp_dict_2 = {}
                                        for row_number in row_numbers:
                                            row_key = 'row_'+str(row_number)
                                            temp_dict_2[row_key] = {'__RequirementLevel__':self.sub_folder_struture[key][sub_key],
                                                                 '__Value__': 'MISSING'}
                                        if temp_dict_2:
                                            temp_dict[sub_key] = temp_dict_2
                                            temp_dict[sub_key]['__RequirementLevel__'] = self.sub_folder_struture[key][sub_key]
                                            temp_dict[sub_key]['__Value__'] = 'INVALID'
                                            
                                        
                                        
                        BIDS_sub_folder_missing_metadata[key] = temp_dict
                        if temp_dict:
                            temp_dict['__RequirementLevel__'] = __RequirementLevel__
                            temp_dict['__Value__'] = 'INVALID'
                            metadata_validation_info[key_name] = temp_dict
                       
                        
                    else:
                        __RequirementLevel__ = self.sub_folder_struture[key]['__RequirementLevel__']
                        metadata_validation_info[key_name] = {'__RequirementLevel__' : __RequirementLevel__,
                                                                 '__exists__' : False,
                                                                }      
                        
        return metadata_validation_info


from setuptools import setup, find_packages
import codecs
import os

VERSION = '0.0.2'
DESCRIPTION = 'fNIRS BIDS Validator'
LONG_DESCRIPTION = 'A package that takes path to the fNIRS BIDS folder and returns metadata validation info'

# Setting up
setup(
    name="fNIRS_BIDS_Validator",
    version=VERSION,
    author="Sreekanth Kura",
    author_email="<sreekanth.kura7@gmail.com>",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=LONG_DESCRIPTION,
    packages=find_packages(),
    install_requires=['pandas'],
    keywords=['python', 'fNIRS-BIDS', 'fNIRS', 'BIDS'],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)


import json
import os

BIDS_raw_folder = { 'dataset_description.json' : { '__RequirementLevel__': 'REQUIRED',
                                                     'Name': 'REQUIRED',
                                                     'BIDSVersion' : 'REQUIRED',
                                                     'HEDVersion' : 'RECOMMENDED',
                                                     'DatasetType': 'RECOMMENDED',
                                                     'License' : 'RECOMMENDED',
                                                     'Authors' : 'OPTIONAL',
                                                     'Acknowledgements' : 'OPTIONAL',
                                                     'HowToAcknowledge' : 'OPTIONAL',
                                                     'Funding' : 'OPTIONAL',
                                                     'EthicsApprovals' : 'OPTIONAL',
                                                     'ReferencesAndLinks' : 'OPTIONAL',
                                                     'DatasetDOI' : 'OPTIONAL',
                                                   },
                         'participants.tsv' : {'__RequirementLevel__': 'RECOMMENDED',
                                               'participant_id':'REQUIRED',
                                               'age':'RECOMMENDED',
                                               'sex':'RECOMMENDED',
                                               'handedness':'RECOMMENDED',
                                              },
                         'README' : {'__RequirementLevel__': 'OPTIONAL'},
                         'CHANGES' : {'__RequirementLevel__': 'OPTIONAL'},
                         'LICENSE' : {'__RequirementLevel__': 'OPTIONAL'}, 
                        }

BIDS_fNIRS_subject_folder = {'_nirs.json' : { '__RequirementLevel__': 'REQUIRED',
                                              'TaskName' : 'REQUIRED',
                                              'SamplingFrequency' : 'REQUIRED',
                                              'NIRSChannelCount' : 'REQUIRED',
                                              'NIRSSourceOptodeCount' : 'REQUIRED',
                                              'NIRSDetectorOptodeCount' : 'REQUIRED',
                                              'InstitutionName' : 'RECOMMENDED',
                                              'InstitutionAddress' : 'RECOMMENDED',
                                              'Manufacturer' : 'RECOMMENDED',
                                              'ManufacturersModelName' : 'RECOMMENDED',
                                              'SoftwareVersions' : 'RECOMMENDED',
                                              'TaskDescription' : 'RECOMMENDED',
                                              'Instructions' : 'RECOMMENDED',
                                              'CogAtlasID' : 'RECOMMENDED',
                                              'CogPOID' : 'RECOMMENDED',
                                              'DeviceSerialNumber' : 'RECOMMENDED',
                                              'RecordingDuration' : 'RECOMMENDED',
                                              'HeadCircumference' : 'RECOMMENDED',
                                              'HardwareFilters' : 'RECOMMENDED',
                                              'SubjectArtefactDescription' : 'RECOMMENDED',
                                              'CapManufacturer' : 'RECOMMENDED',
                                              'CapManufacturersModelName' : 'RECOMMENDED',
                                              'SourceType' : 'RECOMMENDED',
                                              'DetectorType' : 'RECOMMENDED',
                                              'ShortChannelCount' : 'RECOMMENDED',
                                              'NIRSPlacementScheme' : 'RECOMMENDED',
                                              'ACCELChannelCount' : 'CONDITIONAL',
                                              'GYROChannelCount' : 'CONDITIONAL',
                                              'MAGNChannelCount' : 'CONDITIONAL',
                                              },
                             
                             '_events.tsv' : { '__RequirementLevel__': 'REQUIRED',
                                                 'onset': 'REQUIRED',
                                                 'duration' : 'REQUIRED',
                                                 'StimulusPresentation' : 'RECOMMENDED',
                                                 'OperatingSystem' : 'RECOMMENDED',
                                                 'SoftwareName' : 'RECOMMENDED',
                                                 'SoftwareRRID' : 'RECOMMENDED',
                                                 'SoftwareVersion' : 'RECOMMENDED',
                                                 'Code' : 'RECOMMENDED',
                                                 'sample' : 'OPTIONAL',
                                                 'trial_type' : 'OPTIONAL',
                                                 'response_time' : 'OPTIONAL',
                                                 'value' : 'OPTIONAL',
                                                 'HED' : 'OPTIONAL',
                                                 'stim_file' : 'OPTIONAL'
                                               },

                              '_channels.tsv' : { '__RequirementLevel__': 'REQUIRED',
                                                  'name' : 'REQUIRED',
                                                  'type' : 'REQUIRED',
                                                  'source' : 'REQUIRED',
                                                  'detector' : 'REQUIRED',
                                                  'wavelength_nominal' : 'REQUIRED',
                                                  'units' : 'REQUIRED',
                                                  'sampling_frequency' : 'CONDITIONAL',
                                                  'orientation_component' : 'CONDITIONAL',
                                                  'wavelength_actual' : 'OPTIONAL',
                                                  'description' : 'OPTIONAL',
                                                  'wavelength_emission_actual' : 'OPTIONAL',
                                                  'short_channel' : 'OPTIONAL',
                                                  'status' : 'OPTIONAL',
                                                  'status_description' : 'OPTIONAL',
                                                },

                             '_optodes.tsv' : {'__RequirementLevel__': 'OPTIONAL',
                                               'name' : 'REQUIRED',
                                               'type' : 'REQUIRED',
                                               'x' : 'REQUIRED',
                                               'y' : 'REQUIRED',
                                               'z' : 'REQUIRED',
                                               'template_x' : 'CONDITIONAL',
                                               'template_y' : 'CONDITIONAL',
                                               'template_z' : 'CONDITIONAL',
                                               'description' : 'OPTIONAL',
                                               'detector_type' : 'OPTIONAL',
                                               'source_type' : 'OPTIONAL',

                                              },

                             '_coordsystem.json' : {'__RequirementLevel__' : 'CONDITIONAL',
                                                    'NIRSCoordinateSystem' : 'REQUIRED' ,
                                                    'NIRSCoordinateUnits' : 'REQUIRED' ,
                                                    'NIRSCoordinateSystemDescription' : 'CONDITIONAL',
                                                    'NIRSCoordinateProcessingDescription' : 'RECOMMENDED' ,
                                                    'FiducialsCoordinates' : 'RECOMMENDED' ,
                                                    'FiducialsCoordinateSystem' : 'RECOMMENDED' ,
                                                    'FiducialsCoordinateUnits' : 'RECOMMENDED' ,
                                                    'AnatomicalLandmarkCoordinates' : 'RECOMMENDED' ,
                                                    'AnatomicalLandmarkCoordinateSystem' : 'RECOMMENDED' ,
                                                    'AnatomicalLandmarkCoordinateUnits' : 'RECOMMENDED' ,
                                                    'FiducialsCoordinateSystemDescription' : 'CONDITIONAL' ,
                                                    'AnatomicalLandmarkCoordinateSystemDescription' : 'CONDITIONAL',
                                                    'FiducialsDescription' : 'OPTIONAL',
                                                }
                                 }
                                 
with open('BIDS_raw_folder.json', 'w') as outfile:
    json.dump(BIDS_raw_folder, outfile)
    
with open('BIDS_fNIRS_subject_folder.json', 'w') as outfile:
    json.dump(BIDS_fNIRS_subject_folder, outfile)